<?php

/**
 * @file
 * Zyxware Technologies Copyright © 2020.
 *
 * This Copyright is not free software: you can't redistribute it and/or modify.
 */

require 'site_migration_report_support_fun.inc';
require 'site_migration_report_elements.inc';

/**
 * Implements hook_drush_command().
 */
function site_migration_report_drush_command() {
  $items['site_migration_report'] = array(
    'description' => 'Show a migration audit report.',
    'aliases' => array('smr'),
    'arguments' => array(
      'site_name' => '(Optional) sitename.',
      'format' => '(Optional) format.',
    ),
  );
  return $items;
}

/**
 * Drush command callback for Audit.
 */
function drush_site_migration_report($format = 'json', $site_name = 'undefined') {
  $description_value = smr_upgrade_audit_description();
  $arr_content_index = smr_index_array($description_value);

  $arr_report = array();
  smr_call_elements_function($arr_content_index, $arr_report, $format);
  smr_render_report($arr_report, $format);
}

/**
 * Callback for Audit batch UI.
 */
function upgrade_audit_ui_generate_report($format = 'json', $function, &$context) {
  $path = variable_get('file_private_path');
  if (empty($path)) {
    $path = variable_get('file_public_path', 'sites/default/files');
  }
  if (!file_exists(DRUPAL_ROOT . '/' . $path . '/audit-report')) {
    mkdir(DRUPAL_ROOT . '/' . $path . '/audit-report');
  }
  $filename = DRUPAL_ROOT . '/' . $path . '/audit-report/upgrade-audit-report.md';
  $function_name = $function[0];
  $command = 'drush php-eval "upgrade_audit_ui_generate_section(' . $function_name . ')" >> ' . $filename . ' ';
  shell_exec($command);
  $context['message'] = 'Get Details of ' . $function[1];
}

/**
 * Generate each section for Audit batch UI.
 */
function upgrade_audit_ui_generate_section($function_name) {
  $description_value = smr_upgrade_audit_description();
  $arr_content_index = smr_index_array($description_value);
  $arr_report = array();
  $function_data = array($function_name => $arr_content_index[$function_name]);
  smr_call_elements_function($function_data, $arr_report, 'md');
  $format = 'md';
  smr_render_report($arr_report, $format);
}

/**
 * Call all the element functions.
 */
function smr_call_elements_function($arr_content_index, &$arr_report, $format, $stage = 0) {
  foreach ($arr_content_index as $function_name => $data) {
    if (!empty($data['wrapper'])) {
      $arr_report[] = array(
        array(
          'template_type' => 'wrapper_start',
          'class' => $data['wrapper'],
        ),
      );
    }
    $arr_report_description = array();
    $arr_report_description = array(
      array(
        'template_type' => 'heading',
        'stage' => $stage,
        'data' => $data['title'],
        'id' => $function_name,
      ),
      array(
        'template_type' => 'description',
        'data' => $data['description'],
      ),
      array(
        'template_type' => 'what_means',
        'what_means' => $data['what_means'],
      ),
      array(
        'template_type' => 'read_more',
        'read_more' => $data['read_more'],
      ),
    );
    $arr_report[] = $arr_report_description;
    if (function_exists($function_name)) {
      $return_result = $function_name($arr_report);
      if ($return_result === FALSE) {
        $arr_report[] = array(
          array(
            'template_type' => 'paragraph',
            'data' => "No detail found.\n",
          ),
        );
      }
    }
    if (!empty($data['sub_elements'])) {
      smr_call_elements_function($data['sub_elements'], $arr_report, $format, ($stage + 1));
    }
    if (!empty($data['wrapper'])) {
      $arr_report[] = array(
        array(
          'template_type' => 'wrapper_end',
          'class' => $function_name,
        ),
      );
    }
  }
}

/**
 * Render the report.
 */
function smr_render_report($arr_report, $format = 'json') {

  foreach ($arr_report as $arr_report_element) {
    $template_fun_name = 'smr_' . $format . '_template';
    $template_fun_name($arr_report_element);
  }
}

/**
 * Create report in md format.
 */
function smr_md_template($arr_report_element) {
  foreach ($arr_report_element as $arr_report_element_data) {
    switch ($arr_report_element_data['template_type']) {

      case 'heading':
        $stage_heading = '';
        for ($counter = 0; $counter <= $arr_report_element_data['stage']; $counter++) {
          $stage_heading .= '#';
        }
        $stage_heading .= ' ' . $arr_report_element_data['data'] . '{#' . $arr_report_element_data['id'] . '}';
        drush_print($stage_heading);
        break;

      case 'description':
        $description = '######' . $arr_report_element_data['data'];
        drush_print($description);
        break;

      case 'what_means':
        if ($arr_report_element_data['what_means']) {
          drush_print('**What this means: **' . $arr_report_element_data['what_means']);
        }
        break;

      case 'read_more':
        if ($arr_report_element_data["read_more"]) {
          drush_print('[Read More](' . $arr_report_element_data["read_more"] . ')');
          drush_print("```");
          drush_print("```");
        }
        break;

      case 'list':
        break;

      case 'wrapper_start':
        $wrapper_text = "<div markdown='1' class='" . $arr_report_element_data['class'] . "'>";
        drush_print($wrapper_text);
        break;

      case 'wrapper_end':
        drush_print("</div>");
        break;

      case 'line_with_value':

        if (empty($arr_report_element_data['value'])) {
          $arr_report_element_data['value'] = '-';
        }
        drush_print("```");
        drush_print($arr_report_element_data['label'] . ": " . $arr_report_element_data['value']);
        drush_print("```");
        break;

      case 'line_with_value_list':
        drush_print("```");
        foreach ($arr_report_element_data['data'] as $row_line) {
          if (empty($row_line['value'])) {
            $row_line['value'] = '-';
          }
          drush_print($row_line['label'] . ": " . $row_line['value']);
        }
        drush_print("```");
        break;

      case 'summary_list':
        drush_print("```");
        foreach ($arr_report_element_data['data'] as $row_line) {
          if (empty($row_line['value'])) {
            $row_line['value'] = '-';
          }
          drush_print($row_line['label'] . ": " . $row_line['value']);
        }
        drush_print("```");
        break;

      case 'paragraph':
        drush_print($arr_report_element_data['data']);
        break;

      case 'table':
        $table = new TextTable($arr_report_element_data['row_head'], $arr_report_element_data['data']);
        drush_print($table->render());
        drush_print("```");
        if ($arr_report_element_data['lines_of_code']) {
          drush_print("Total lines of custom code: " . ($arr_report_element_data['lines_of_code']));
        }
        else {
          drush_print("Total records: " . count($arr_report_element_data['data']));
        }
        drush_print("```");
        break;
    }
  }

}

/**
 * Create report in json format.
 */
function smr_json_template($arr_report_element) {
  $arr_site_report = smr_get_general_site_detail();
  $json_site_audit_report = json_encode($arr_site_report);
  drush_print($json_site_audit_report);
}

/**
 * Create description and read more link for each sections.
 */
function smr_upgrade_audit_description() {
  return array(
    'smr_summary' => array(
      'what' => "D7 Audit is a free, open source Drupal 7 administration module giving quick access to key site information and features.",
      'what_means' => "Easily upgrade your Drupal website from Drupal 7 to 9. This platform makes upgrading quick and easy, allowing you to focus on your business goals rather than the IT.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_basic_information' => array(
      'what' => "General information about this site, eg. showing the current version of drupal that is installed.",
      'what_means' => "See a summary of your current Drupal 7 website setup.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_content' => array(
      'what' => "Information that is presented on the site.",
      'what_means' => "Content is a core part of every website and that's where D7 Upgrade comes into play. It allows you to create, customize and reuse content in a variety of ways. Whether you want to bundle it for search engines or create a custom design for each user role, these tools will help you create content that works for your users and helps your business grow.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_content_types' => array(
      'what' => "Details on predefined structures for information on the website. A single site could contain many types of content, such as informational pages, news items, polls, blog posts, real estate listings, etc.",
      'what_means' => "A single website can contain many types of content. With predefined structures for information, you can easily add a new type of content to the site with a few clicks.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_nodes' => array(
      'what' => "Summary information on the individual peices of content that exist on this site.",
      'what_means' => "Looking for information on the individual pieces of content that exist on this website? Develop and enhance your knowledge with free high quality training.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_files' => array(
      'what' => "The official files directory for this website typically includes the PDFs, audio files, images and other media present on the website.",
      'what_means' => "The D7 upgrade allows you to upload files faster, store more content and keep all your files together in one place. Get total control over your project, with the power to create new projects, organize files and upload large collections of media.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_webforms' => array(
      'what' => "Forms on the site that collect data that is submitted by site visitors.",
      'what_means' => "Engage your customers with Webforms on the site that collect data that is submitted by site visitors. This can be an order form, a contact form, or any other custom feature you want to add.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_taxonomy' => array(
      'what' => "Taxonomies are used to classify website information and allow you to connect, relate and classify your website's content.",
      'what_means' => "Taxonomies are used to classify website information and allow you to connect, relate and classify your website's content. Taxonomies can help you organize your website's content so that it's easy for both humans and computers to browse, find and understand your site.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_views' => array(
      'what' => "Dynamically generated listings of information on the website. Views govern how content is presented.",
      'what_means' => "Views govern how content is presented, and are the most powerful way of presenting data in Drupal. They allow for complex, dynamic and flexible ways to create content pages.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_blocks' => array(
      'what' => "A grouping of all the available boxes of content rendered into a specific area, or region.",
      'what_means' => "Blocks can be used to create almost any kind of content on the page. Blocks are useful when you need to display similar content on multiple pages or in different positions.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_list_of_enabled_blocks' => array(
      'what' => "Boxes of content that are currently actively rendered into a specific area, or region on the website.",
      'what_means' => "This upgrade audit helps you easily identify and track the number of boxes that are currently being rendered on your website.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_list_of_enabled_region' => array(
      'what' => "Areas that are built in to the website's design. These are typically used to place and position specific content and blocks.",
      'what_means' => "Upgrade your website's design with countless design elements and options, available to be placed and positioned right in the areas that are built in to the website's design.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_modules' => array(
      'what' => "Functional components and features that include all the logic that is at work in your website.",
      'what_means' => "Unlock your website's full potential by including our D7 upgrade audit report. Functionality will become easier, richer experiences will be built, and all will be improved through the use of features like modules, custom post types and user management.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_contributed_modules' => array(
      'what' => "Details on modules that are available to download from <a target = _blank href = 'https://www.drupal.org'>drupal.org</a>.",
      'what_means' => " <a target = _blank href = 'https://www.drupal.org'>drupal.org</a> includes a wide variety of modules that you can add to your Drupal website. Contributed by the Drupal community and designed to extend core functionality and provide new features.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_custom_modules' => array(
      'what' => "Modules that were developed privately to achieve specific website goals.",
      'what_means' => "Custom modules are created clients who wanted to use Drupal in a more advanced, tailored way. Custom modules are designed to achieve specific website goals, and will give you the control you need to get the most ouf of your website.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_core_modules' => array(
      'what' => "Drupal modules that are available to all websites by default.",
      'what_means' => "Drupal core modules provide the basic functionality to create websites. These are available to all sites by default, and can be used in your own custom modules to ensure consistent behavior regardless of customization.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_features' => array(
      'what' => "List of features available for this website.",
      'what_means' => "The D7 Upgrade can increase your website's speed and stability, as well as add new features to enhance the experience of your visitors.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_additional_information' => array(
      'what' => "These are some more details to consider in your upgrade process.",
      'what_means' => "The D7 Upgrade process is easy, safe and straightforward. Just follow the steps below to get started.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_theme' => array(
      'what' => "The overall look and feel of your website is governed by the a Drupal theme. ",
      'what_means' => "A Drupal theme lets you change the overall look and feel of your website. Drupal themes are templates that control how your site appears to visitors, and offer more design options than you're used to in most platforms.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_list_of_templates_in_the_custom_theme' => array(
      'what' => "Repeated or reused sections of the website that have a specially specified look or layout.",
      'what_means' => "A more consistent look and feel on your website, between sections, helps your visitors understand where they are in the site and makes it easy to recognise the difference between different kinds of content.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_list_of_js_files_included' => array(
      'what' => "Javascript files that are included in your website.",
      'what_means' => "Get the convenience of a one-click upgrade to the latest version of Drupal.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_list_of_css_files_included' => array(
      'what' => "Code that governs the overall styling of your website.",
      'what_means' => "Moving from one CSS framework to another can be difficult. We want our customers to feel confident in the process and with their website. This is why we offer a free clean up of your current code and migrate it over to Drupal 9.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_templates_php' => array(
      'what' => "This is where additional functionality often resides in Drupal 7 websites. This may need to be ported to Drupal 9.",
      'what_means' => "The D7 Upgrade module allows administrators to convert features that are still in Drupal 7 into Drupal 9 modules. This feature steps you through the process of identifying and describing which of your existing components need to be ported, then creates a list of candidate modules for each component so that developers can start working on them.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_list_of_functions_used' => array(
      'what' => "Functionality defined in the theme layer.",
      'what_means' => "The D7 Upgrade lets you customize the functionality of your website to fit your brand and create a more engaging web experience.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_users' => array(
      'what' => "A list of all active users and roles on your website",
      'what_means' => "This upgrade will allow you to see all the active roles and users on your website.",
      'read_more' => "https://d7upgrade.org",
    ),
    'smr_crons' => array(
      'what' => "Website tasks that are scheduled to be performed at regular intervals.",
      'what_means' => "Regular website maintenance helps to ensure you don't run into a website disaster. Our D7 Upgrade service includes scheduled tasks, such as database backups, security patches and updates, malware scanning, and more. We'll also do some site analysis to make sure everything is working smoothly and provide recommendations for improvements as needed.",
      'read_more' => "https://d7upgrade.org",
    ),
  );
}
